/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License;
 * you may not use this file except in compliance with the Elastic License.
 */

import { getOr, isEqual } from 'lodash/fp';
import React, { useCallback } from 'react';
import { connect } from 'react-redux';
import { Dispatch } from 'redux';

import { esFilters, IIndexPattern } from '../../../../../../../../src/plugins/data/public';
import { BrowserFields } from '../../../containers/source';
import { convertKueryToElasticSearchQuery } from '../../../lib/keury';
import {
  KueryFilterQuery,
  SerializedFilterQuery,
  State,
  timelineSelectors,
  inputsModel,
  inputsSelectors,
} from '../../../store';
import { timelineActions } from '../../../store/actions';
import { KqlMode, TimelineModel, EventType } from '../../../store/timeline/model';
import { timelineDefaults } from '../../../store/timeline/defaults';
import { DispatchUpdateReduxTime, dispatchUpdateReduxTime } from '../../super_date_picker';
import { DataProvider } from '../data_providers/data_provider';
import { SearchOrFilter } from './search_or_filter';

interface OwnProps {
  browserFields: BrowserFields;
  indexPattern: IIndexPattern;
  timelineId: string;
}

interface StateReduxProps {
  dataProviders: DataProvider[];
  eventType: EventType;
  filters: esFilters.Filter[];
  filterQuery: KueryFilterQuery;
  filterQueryDraft: KueryFilterQuery;
  from: number;
  fromStr: string;
  isRefreshPaused: boolean;
  kqlMode: KqlMode;
  refreshInterval: number;
  savedQueryId: string | null;
  to: number;
  toStr: string;
}

interface DispatchProps {
  applyKqlFilterQuery: ({
    id,
    filterQuery,
  }: {
    id: string;
    filterQuery: SerializedFilterQuery;
  }) => void;
  updateEventType: ({ id, eventType }: { id: string; eventType: EventType }) => void;
  updateKqlMode: ({ id, kqlMode }: { id: string; kqlMode: KqlMode }) => void;
  setKqlFilterQueryDraft: ({
    id,
    filterQueryDraft,
  }: {
    id: string;
    filterQueryDraft: KueryFilterQuery;
  }) => void;
  setSavedQueryId: ({ id, savedQueryId }: { id: string; savedQueryId: string | null }) => void;
  setFilters: ({ id, filters }: { id: string; filters: esFilters.Filter[] }) => void;
  updateReduxTime: DispatchUpdateReduxTime;
}

type Props = OwnProps & StateReduxProps & DispatchProps;

const StatefulSearchOrFilterComponent = React.memo<Props>(
  ({
    applyKqlFilterQuery,
    browserFields,
    dataProviders,
    eventType,
    filters,
    filterQuery,
    filterQueryDraft,
    from,
    fromStr,
    indexPattern,
    isRefreshPaused,
    kqlMode,
    refreshInterval,
    savedQueryId,
    setFilters,
    setKqlFilterQueryDraft,
    setSavedQueryId,
    timelineId,
    to,
    toStr,
    updateEventType,
    updateKqlMode,
    updateReduxTime,
  }) => {
    const applyFilterQueryFromKueryExpression = useCallback(
      (expression: string, kind) =>
        applyKqlFilterQuery({
          id: timelineId,
          filterQuery: {
            kuery: {
              kind,
              expression,
            },
            serializedQuery: convertKueryToElasticSearchQuery(expression, indexPattern),
          },
        }),
      [indexPattern, timelineId]
    );

    const setFilterQueryDraftFromKueryExpression = useCallback(
      (expression: string, kind) =>
        setKqlFilterQueryDraft({
          id: timelineId,
          filterQueryDraft: {
            kind,
            expression,
          },
        }),
      [timelineId]
    );

    const setFiltersInTimeline = useCallback(
      (newFilters: esFilters.Filter[]) =>
        setFilters({
          id: timelineId,
          filters: newFilters,
        }),
      [timelineId]
    );

    const setSavedQueryInTimeline = useCallback(
      (newSavedQueryId: string | null) =>
        setSavedQueryId({
          id: timelineId,
          savedQueryId: newSavedQueryId,
        }),
      [timelineId]
    );

    const handleUpdateEventType = useCallback(
      (newEventType: EventType) =>
        updateEventType({
          id: timelineId,
          eventType: newEventType,
        }),
      [timelineId]
    );

    return (
      <SearchOrFilter
        applyKqlFilterQuery={applyFilterQueryFromKueryExpression}
        browserFields={browserFields}
        dataProviders={dataProviders}
        eventType={eventType}
        filters={filters}
        filterQuery={filterQuery}
        filterQueryDraft={filterQueryDraft}
        from={from}
        fromStr={fromStr}
        indexPattern={indexPattern}
        isRefreshPaused={isRefreshPaused}
        kqlMode={kqlMode!}
        refreshInterval={refreshInterval}
        savedQueryId={savedQueryId}
        setFilters={setFiltersInTimeline}
        setKqlFilterQueryDraft={setFilterQueryDraftFromKueryExpression!}
        setSavedQueryId={setSavedQueryInTimeline}
        timelineId={timelineId}
        to={to}
        toStr={toStr}
        updateEventType={handleUpdateEventType}
        updateKqlMode={updateKqlMode!}
        updateReduxTime={updateReduxTime}
      />
    );
  },
  (prevProps, nextProps) => {
    return (
      prevProps.eventType === nextProps.eventType &&
      prevProps.from === nextProps.from &&
      prevProps.fromStr === nextProps.fromStr &&
      prevProps.to === nextProps.to &&
      prevProps.toStr === nextProps.toStr &&
      prevProps.isRefreshPaused === nextProps.isRefreshPaused &&
      prevProps.refreshInterval === nextProps.refreshInterval &&
      prevProps.timelineId === nextProps.timelineId &&
      isEqual(prevProps.browserFields, nextProps.browserFields) &&
      isEqual(prevProps.dataProviders, nextProps.dataProviders) &&
      isEqual(prevProps.filters, nextProps.filters) &&
      isEqual(prevProps.filterQuery, nextProps.filterQuery) &&
      isEqual(prevProps.filterQueryDraft, nextProps.filterQueryDraft) &&
      isEqual(prevProps.indexPattern, nextProps.indexPattern) &&
      isEqual(prevProps.kqlMode, nextProps.kqlMode) &&
      isEqual(prevProps.savedQueryId, nextProps.savedQueryId) &&
      isEqual(prevProps.timelineId, nextProps.timelineId)
    );
  }
);
StatefulSearchOrFilterComponent.displayName = 'StatefulSearchOrFilterComponent';

const makeMapStateToProps = () => {
  const getTimeline = timelineSelectors.getTimelineByIdSelector();
  const getKqlFilterQueryDraft = timelineSelectors.getKqlFilterQueryDraftSelector();
  const getKqlFilterQuery = timelineSelectors.getKqlFilterKuerySelector();
  const getInputsTimeline = inputsSelectors.getTimelineSelector();
  const getInputsPolicy = inputsSelectors.getTimelinePolicySelector();
  const mapStateToProps = (state: State, { timelineId }: OwnProps) => {
    const timeline: TimelineModel = getTimeline(state, timelineId) ?? timelineDefaults;
    const input: inputsModel.InputsRange = getInputsTimeline(state);
    const policy: inputsModel.Policy = getInputsPolicy(state);
    return {
      dataProviders: timeline.dataProviders,
      eventType: timeline.eventType ?? 'raw',
      filterQuery: getKqlFilterQuery(state, timelineId),
      filterQueryDraft: getKqlFilterQueryDraft(state, timelineId),
      filters: timeline.filters,
      from: input.timerange.from,
      fromStr: input.timerange.fromStr,
      isRefreshPaused: policy.kind === 'manual',
      kqlMode: getOr('filter', 'kqlMode', timeline),
      refreshInterval: policy.duration,
      savedQueryId: getOr(null, 'savedQueryId', timeline),
      to: input.timerange.to,
      toStr: input.timerange.toStr,
    };
  };
  return mapStateToProps;
};

const mapDispatchToProps = (dispatch: Dispatch) => ({
  applyKqlFilterQuery: ({ id, filterQuery }: { id: string; filterQuery: SerializedFilterQuery }) =>
    dispatch(
      timelineActions.applyKqlFilterQuery({
        id,
        filterQuery,
      })
    ),
  updateEventType: ({ id, eventType }: { id: string; eventType: EventType }) =>
    dispatch(timelineActions.updateEventType({ id, eventType })),
  updateKqlMode: ({ id, kqlMode }: { id: string; kqlMode: KqlMode }) =>
    dispatch(timelineActions.updateKqlMode({ id, kqlMode })),
  setKqlFilterQueryDraft: ({
    id,
    filterQueryDraft,
  }: {
    id: string;
    filterQueryDraft: KueryFilterQuery;
  }) =>
    dispatch(
      timelineActions.setKqlFilterQueryDraft({
        id,
        filterQueryDraft,
      })
    ),
  setSavedQueryId: ({ id, savedQueryId }: { id: string; savedQueryId: string | null }) =>
    dispatch(timelineActions.setSavedQueryId({ id, savedQueryId })),
  setFilters: ({ id, filters }: { id: string; filters: esFilters.Filter[] }) =>
    dispatch(timelineActions.setFilters({ id, filters })),
  updateReduxTime: dispatchUpdateReduxTime(dispatch),
});

export const StatefulSearchOrFilter = connect(
  makeMapStateToProps,
  mapDispatchToProps
)(StatefulSearchOrFilterComponent);
